package ij.plugin.filter;
import java.awt.*;
import java.util.*;
import ij.*;
import ij.gui.*;
import ij.process.*;
import ij.text.*;

/** This plug-in implements the Image/Show Info command. */
public class Info implements PlugInFilter {
    private ImagePlus imp;

	public int setup(String arg, ImagePlus imp) {
		this.imp = imp;
		return DOES_ALL+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		String infoProperty = (String)imp.getProperty("Info");
		String info = getInfo(ip);
		if (infoProperty!=null)
			showInfo(infoProperty+"\n------------------------\n"
				+info, 400, 500);
		else
			showInfo(info, 300, 300);
	}

	String getInfo(ImageProcessor ip) {
		String s = new String("\n");
		s += "Title: '" + imp.getTitle() + "'\n";
		if (imp.sCalibrated) {
	    	s += "Width:  "+IJ.d2s(imp.getWidth()*imp.pixelWidth,2)+" " + imp.units+" ("+imp.getWidth()+")\n";
	    	s += "Height:  "+IJ.d2s(imp.getHeight()*imp.pixelHeight,2)+" " + imp.units+" ("+imp.getHeight()+")\n";
	    	if (imp.pixelWidth==imp.pixelHeight)
	    		s += "Resolution:  "+IJ.d2s(1.0/imp.pixelWidth,1) + " pixels per "+imp.unit+"\n";
	    	else {
	    		s += "X Resolution:  "+IJ.d2s(1.0/imp.pixelWidth,1) + " pixels per "+imp.unit+"\n";
	    		s += "Y Resolution:  "+IJ.d2s(1.0/imp.pixelHeight,1) + " pixels per "+imp.unit+"\n";
	    	}
	    } else {
	    	s += "Width:  " + imp.getWidth() + " pixels\n";
	    	s += "Height:  " + imp.getHeight() + " pixels\n";
	    }
	    int type = imp.getType();
    	switch (type) {
	    	case ImagePlus.GRAY8:
	    		s += "Bits per pixel: 8 ";
	    		if (imp.isInvertedLut())
	    			s += "(inverted LUT)\n";
	    		else
	    			s += "(grayscale LUT)\n";
	    		break;
	    	case ImagePlus.GRAY16: case ImagePlus.GRAY32:
	    		if (type==ImagePlus.GRAY16) {
					ShortProcessor sp = (ShortProcessor)imp.getProcessor(); 
	    			if (sp.isUnsigned())
	    				s += "Bits per pixel: 16 (unsigned short)\n";
	    			else
	    				s += "Bits per pixel: 16 (signed short)\n";
	    		} else
	    			s += "Bits per pixel: 32 (float)\n";
		    		s += "Display window: " + IJ.d2s(ip.getMin()) + " - " + IJ.d2s(ip.getMax()) + "\n";
	    		break;
	    	case ImagePlus.COLOR_256:
	    		s += "Bits per pixel: 8 (color LUT)\n";
	    		break;
	    	case ImagePlus.COLOR_RGB:
	    		s += "Bits per pixel: 32 (RGB)\n";
	    		break;
    	}
    	int nSlices = imp.getStackSize();
    	if (nSlices>1)
			s += "Slices: " + nSlices + " (" + imp.getCurrentSlice() + ")\n";
		if (ip.getMinThreshold()==ip.NO_THRESHOLD)
	    	s += "No Threshold\n";
	    else
			s += "Threshold: "+IJ.d2s(ip.getMinThreshold(),0)+"-"+IJ.d2s(ip.getMaxThreshold(),0)+"\n";
		ImageCanvas ic = imp.getWindow().getCanvas();
    	double mag = ic.getMagnification();
    	if (mag!=1.0)
			s += "Magnification: " + mag + "\n";
	    Roi roi = imp.getRoi();
	    if (roi == null)
	    	s += "No ROI\n";
	    else {
    		switch (roi.getType()) {
    			case Roi.RECTANGLE: s += "Rectangular ROI\n"; break;
    			case Roi.OVAL: s += "Oval ROI\n"; break;
    			case Roi.POLYGON: s += "Polygon ROI\n"; break;
    			case Roi.FREEROI: s += "Freehand ROI\n"; break;
    			case Roi.LINE: s += "Line Selection\n"; break;
    			case Roi.POLYLINE: s += "Polyline Selection\n"; break;
    			case Roi.FREELINE: s += "Freehand line Selection\n"; break;
    		}
	    	Rectangle r = roi.getBoundingRect();
			if (imp.sCalibrated) {
				s += "  X: " + IJ.d2s(r.x*imp.pixelWidth,2) + " (" + r.x + ")\n";
				s += "  Y: " + IJ.d2s(r.y*imp.pixelHeight,2) + " (" +  r.y + ")\n";
				s += "  Width: " + IJ.d2s(r.width*imp.pixelWidth,2) + " (" +  r.width + ")\n";
				s += "  Height: " + IJ.d2s(r.height*imp.pixelHeight,2) + " (" +  r.height + ")\n";
			} else {
				s += "  X: " + r.x + "\n";
				s += "  Y: " + r.y + "\n";
				s += "  Width: " + r.width + "\n";
				s += "  Height: " + r.height + "\n";
	    	}
	    }

		s += " \n";
		s += IJ.freeMemory() + "\n";
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		s += "Screen Size: " + d.width + "x" + d.height;
		return s;
	}

	void showInfo(String info, int width, int height) {
		TextWindow tw = new TextWindow("Info for "+imp.getTitle(), info, width, height);
	}

}
